#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.association.interfaces import IAssociationContainer, IAssociationTarget, \
    ASSOCIATION_CONTAINER_KEY, IAssociationInfo
from zope.annotation.interfaces import IAnnotations
from zope.location.interfaces import ISublocations
from zope.traversing.interfaces import ITraversable

# import packages
from pyams_catalog.utils import index_object
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from pyramid.threadlocal import get_current_registry
from zope.container.ordered import OrderedContainer
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


@implementer(IAssociationContainer)
class AssociationContainer(OrderedContainer):
    """Associations container"""

    last_id = 1

    def append(self, value, notify=True):
        key = str(self.last_id)
        if not notify:
            # pre-locate association item to avoid multiple notifications
            locate(value, self, key)
        self[key] = value
        self.last_id += 1
        if not notify:
            # make sure that association item is correctly indexed
            index_object(value)


@adapter_config(context=IAssociationTarget, provides=IAssociationContainer)
def association_container_factory(target):
    """Associations container factory"""
    annotations = IAnnotations(target)
    container = annotations.get(ASSOCIATION_CONTAINER_KEY)
    if container is None:
        container = annotations[ASSOCIATION_CONTAINER_KEY] = AssociationContainer()
        get_current_registry().notify(ObjectCreatedEvent(container))
        locate(container, target, '++ass++')
    return container


@adapter_config(name='ass', context=IAssociationTarget, provides=ITraversable)
class AssociationContainerNamespace(ContextAdapter):
    """Associations container ++association++ namespace"""

    def traverse(self, name, furtherpath=None):
        return IAssociationContainer(self.context)


@adapter_config(name='associations', context=IAssociationTarget, provides=ISublocations)
class AssociationContainerSublocations(ContextAdapter):
    """Associations container sub-locations adapter"""

    def sublocations(self):
        return IAssociationContainer(self.context).values()


@vocabulary_config(name='PyAMS content associations')
class ContentAssociationsVocabulary(SimpleVocabulary):
    """Content associations vocabulary"""

    def __init__(self, context=None):
        terms = []
        target = get_parent(context, IAssociationTarget)
        if target is not None:
            terms = [SimpleTerm(link.__name__, title=IAssociationInfo(link).inner_title)
                     for link in IAssociationContainer(target).values()]
        super(ContentAssociationsVocabulary, self).__init__(terms)
